"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = __importDefault(require("../util/Logger"));
/*
 * Base Docker Command
 */
class AbstractDockerCommand {
    constructor() {
        this.imageExists = false;
        this.openapiGeneratorImage = "openapitools/openapi-generator-cli:v4.2.2";
    }
    checkDockerImage(docker) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                var opts = { "filters": '{"reference": ["' + this.openapiGeneratorImage + '"]}' };
                Logger_1.default.i("Calling docker.listImages()");
                yield docker.listImages(opts, function (error, addresses) {
                    Logger_1.default.i("docker.listImages() error is " + error);
                    if (error === null) {
                        if (addresses && addresses.length > 0) {
                            try {
                                Logger_1.default.i("First image is: " + JSON.stringify(addresses[0]));
                            }
                            catch (e) {
                                // Ignore
                            }
                            resolve(true);
                        }
                        else {
                            reject(false);
                        }
                    }
                    else {
                        reject(false);
                    }
                });
            }));
        });
    }
    pullDockerImage(docker, progress) {
        return __awaiter(this, void 0, void 0, function* () {
            Logger_1.default.i("Calling docker.checkAuth()");
            // Check auth first.  Should be successful.          
            var authCheck = yield this.checkAuth(docker).then((result) => __awaiter(this, void 0, void 0, function* () {
                Logger_1.default.i("Resulting Auth is: " + result);
                return true;
            })).catch((result) => __awaiter(this, void 0, void 0, function* () {
                Logger_1.default.i("Resulting Auth is: " + result);
                return false;
            }));
            Logger_1.default.i("Calling docker.pull() after authCheck: " + authCheck);
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                yield docker.pull(this.openapiGeneratorImage, (error, streamo) => __awaiter(this, void 0, void 0, function* () {
                    // Log the output of pull.
                    // streamo.on('data', (d: any) => {
                    //    Log.i(d.toString());
                    // });
                    Logger_1.default.i("Docker pull error: " + error);
                    docker.modem.followProgress(streamo, (er, output) => __awaiter(this, void 0, void 0, function* () {
                        // onFinished() callback
                        // Seems to be the only way to 'wait' until the image is finished pulling
                        // prior to invoking code gen
                        if (er) {
                            Logger_1.default.e("docker.pull error = " + er);
                            reject();
                        }
                        else {
                            Logger_1.default.i("docker.pull has no error");
                        }
                        output.forEach((a) => {
                            if (a) {
                                // Log.i(JSON.stringify(a));            
                            }
                        });
                        if (er === null) {
                            // this.doGeneration();
                            resolve();
                        }
                        else {
                            Logger_1.default.e("Docker pull error: " + er);
                            reject();
                        }
                    }), (event) => __awaiter(this, void 0, void 0, function* () {
                        // onProgress() callback            
                        progress.report({ message: event.status });
                    }));
                }));
            }));
        });
    }
    checkAuth(docker) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(function (resolve, reject) {
                return __awaiter(this, void 0, void 0, function* () {
                    yield docker.checkAuth({ serveraddress: 'https://index.docker.io/' }, (err, responseSchema) => __awaiter(this, void 0, void 0, function* () {
                        // To track auth progress
                        // docker.modem.followProgress(responseSchema, onFinished, onProgress);
                        // function onFinished(err: any, output: any) {
                        // }
                        // function onProgress(event: any) {
                        // }
                        // // OR 
                        // docker.modem.followProgress(responseSchema, (err: any, output: any) => {
                        //     // onFinished
                        // }, (event: any) => {
                        //     // onProgress
                        // });
                        try {
                            if (responseSchema !== null) {
                                Logger_1.default.i("IdentityToken:" + responseSchema.IdentityToken);
                                Logger_1.default.i("Status:" + responseSchema.Status);
                            }
                        }
                        catch (e) {
                            // Ignore
                        }
                        if (responseSchema.Status === "Login Succeeded" || err === null) {
                            resolve(true);
                        }
                        if (err !== null) {
                            Logger_1.default.e("Checking authentication to dockerhub server failed.");
                            Logger_1.default.e("- Reason:" + err.reason);
                            Logger_1.default.e("- StatusCode:" + err.statusCode);
                            Logger_1.default.e("- Message:" + err.json.message);
                        }
                        reject(false);
                    }));
                });
            });
        });
    }
}
exports.default = AbstractDockerCommand;
//# sourceMappingURL=AbstractDockerCommand.js.map