"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
const vscode = __importStar(require("vscode"));
const Constants_1 = __importDefault(require("../constants/Constants"));
const Logger_1 = __importDefault(require("../util/Logger"));
const stream_1 = require("stream");
const AbstractDockerCommand_1 = __importDefault(require("./AbstractDockerCommand"));
const Translator_1 = __importDefault(require("../constants/Translator"));
var fs = require('fs');
var dockerode = require('dockerode');
class AbstractGenerateCommand extends AbstractDockerCommand_1.default {
    constructor(_generatorType) {
        super();
        this._generatorType = _generatorType;
        this.projectName = "";
        this.localPath = vscode.Uri.parse("file://");
        this.selectedDefinition = "";
        this.fqPathToDefinition = "";
        this.fqPathOutputLocation = "";
        this.preferredSourceLocation = "";
        this.projectLanguage = "";
        this.pathSeparator = "";
        this.projectType = "";
        this.selectedUriFolder = vscode.Uri.parse("file://");
        this.pathSeparator = this.getPathSeparator();
    }
    analyzeSelection(selection) {
        return __awaiter(this, void 0, void 0, function* () {
            if (selection !== undefined) {
                if (Reflect.has(selection, "name")) {
                    this.projectName = Reflect.get(selection, "name");
                }
                if (Reflect.has(selection, "localPath")) {
                    this.localPath = Reflect.get(selection, "localPath");
                }
                if (Reflect.has(selection, "type")) {
                    var typeField = Reflect.get(selection, "type");
                    if (Reflect.has(typeField, "internalType")) {
                        this.projectType = Reflect.get(typeField, "internalType");
                    }
                }
                Logger_1.default.i("Selected project is: " + this.projectName);
                Logger_1.default.i("Selected projectType is: " + this.projectType);
                Logger_1.default.i("Selected project local path is: " + this.localPath.fsPath + " , " + this.localPath.path);
            }
        });
    }
    promptForProject() {
        return __awaiter(this, void 0, void 0, function* () {
            let workspaceFolders = vscode.workspace.workspaceFolders;
            var workspaces = [];
            var workspaceNames = [];
            if (workspaceFolders) {
                var folders = [];
                let workspaceFolderContents;
                var fullFolderPaths = [];
                for (var i = 0; i < workspaceFolders.length; i++) {
                    workspaces.push(workspaceFolders[i]);
                    workspaceNames.push(workspaceFolders[i].name);
                    var wsFolder = workspaceFolders[i];
                    var filePath = wsFolder.uri.fsPath;
                    workspaceFolderContents = fs.readdirSync(filePath, { withFileTypes: true });
                    workspaceFolderContents.forEach((ff) => {
                        var fsdir = ff;
                        try {
                            var name = (fsdir.name !== null && fsdir.name !== undefined) ? fsdir.name.toString() : fsdir.toString();
                            if (fs.lstatSync(filePath + this.pathSeparator + name).isDirectory() && !name.startsWith('.') && name !== "bin") {
                                folders.push(name);
                                fullFolderPaths.push(wsFolder.uri.fsPath.toString() + this.pathSeparator + name);
                            }
                        }
                        catch (error) {
                            Logger_1.default.e("ERROR : " + error);
                        }
                    });
                }
                if (folders.length > 0) {
                    yield vscode.window.showQuickPick(folders, Constants_1.default.QUICK_PICK_OPTION_PROJECT_FOLDERS).then((selectedFolder) => __awaiter(this, void 0, void 0, function* () {
                        if (selectedFolder === undefined) {
                            return;
                        }
                        var idx = folders.indexOf(selectedFolder);
                        this.localPath = vscode.Uri.file(fullFolderPaths[idx]);
                        this.projectName = selectedFolder;
                    }));
                    if (this.projectName === "") {
                        return;
                    }
                    var breakForEach = {};
                    try {
                        filePath = this.localPath.fsPath;
                        var dotProjectFolder = null;
                        try {
                            dotProjectFolder = fs.readdirSync(filePath + "/../.projects", { withFileTypes: true });
                        }
                        catch (projectError) {
                            // ignore
                        }
                        if (dotProjectFolder) {
                            dotProjectFolder.forEach((projInfFile) => {
                                var name = (projInfFile.name !== null && projInfFile.name !== undefined) ? projInfFile.name.toString() : projInfFile.toString();
                                var contents = fs.readFileSync(filePath + "/../.projects/" + name);
                                var jsonObject = JSON.parse(contents);
                                if (this.projectName === jsonObject.name) {
                                    this.projectLanguage = jsonObject.language;
                                    throw breakForEach;
                                }
                            });
                        }
                    }
                    catch (err) {
                        // ignore if there is no .projects folder potentially MC's
                        if (err === breakForEach) {
                            Logger_1.default.i("Language from meta data is " + this.projectLanguage);
                        }
                        else {
                            Logger_1.default.e("Error parsing project inf file: " + err);
                        }
                    }
                }
                else {
                    yield vscode.window.showErrorMessage(Translator_1.default.getString("wizard.errorNoFoldersInWorkspace"));
                    return;
                }
            }
        });
    }
    promptForOpenApiDefinition() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                var openDialogOptions = {
                    canSelectFiles: true,
                    canSelectFolders: false,
                    canSelectMany: false,
                    defaultUri: this.localPath,
                    filters: { [`${Translator_1.default.getString("wizard.openApiFilter")}`]: ['yaml', 'yml', 'json'] },
                    openLabel: Translator_1.default.getString("wizard.promptForDefinition") // Has to be short, for the 'button'
                };
                var selectedFile = [];
                selectedFile = yield vscode.window.showOpenDialog(openDialogOptions).then((tempSelectedFile) => {
                    Logger_1.default.i("Selected OpenAPI Definition is = " + tempSelectedFile);
                    if (tempSelectedFile === undefined) {
                        return [];
                    }
                    return tempSelectedFile;
                });
                if (selectedFile.length === 0) {
                    reject("Selected file was not selected."); // For log
                    return; // Dialog was canceled
                }
                // ***** -i option ***
                this.selectedDefinition = this.getFilenameFromPath(this.localPath.path, selectedFile[0].path);
                // ************************************
                Logger_1.default.i("Selected file is: " + this.selectedDefinition);
                if (this.selectedDefinition.length === 0) {
                    // Do this because the file browser allows users to navigate to locations outside of the workspace
                    // and potentially select a file from a different folder. We need to volume mount the project path, 
                    // based on the selected project from the previous step during this command flow
                    vscode.window.showErrorMessage(Translator_1.default.getString("wizard.errorSelectOnlyFromProject", this.localPath.path));
                    return;
                }
                if (!this.isPotentialOpenApiFile(this.selectedDefinition)) {
                    vscode.window.showErrorMessage(Translator_1.default.getString("wizard.errorNotADefinition", this.selectedDefinition));
                    return;
                }
                // Mount point
                this.fqPathToDefinition = this.getPlatformPath(this.localPath.path); // NOT fsPath
                resolve();
            }));
        });
    }
    getOutputLocation() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                var openDialogOptions = {
                    canSelectFiles: false,
                    canSelectFolders: true,
                    canSelectMany: false,
                    defaultUri: this.preferredSourceLocation.length === 0 ? this.localPath : vscode.Uri.file(this.localPath.fsPath.toString() + this.preferredSourceLocation),
                    openLabel: Translator_1.default.getString("wizard.selectOutputFolder") // Has to be short, for the 'button'
                };
                var selectedWSFolder = [];
                selectedWSFolder = yield vscode.window.showOpenDialog(openDialogOptions).then((tempSelectedWSFolder) => {
                    Logger_1.default.i("Selected workspace folder output location = " + tempSelectedWSFolder);
                    if (tempSelectedWSFolder === undefined) {
                        return [];
                    }
                    return tempSelectedWSFolder;
                });
                if (selectedWSFolder.length === 0) {
                    reject("Output folder was not selected."); // For log
                    return; // Dialog was canceled
                }
                // Do this because the file browser allows users potentially navigate to outside of the workspace
                // and select a file from a different folder.  We need to volume mount this path, from the selected
                // project in the previous step in this command flow
                var isChildFolder = this.isChildFolder(this.localPath.path, selectedWSFolder[0].path);
                if (!isChildFolder) {
                    vscode.window.showErrorMessage(Translator_1.default.getString("wizard.errorSelectFolderFromProject", this.localPath.path));
                    return;
                }
                // ***** -o Generator output option ***
                this.selectedUriFolder = selectedWSFolder[0];
                this.fqPathOutputLocation = this.getPlatformPath(this.selectedUriFolder.path);
                // ************************************   
                try {
                    Logger_1.default.i("Check .openapi-generator-ignore file:" + this.selectedUriFolder.fsPath + this.pathSeparator + ".openapi-generator-ignore");
                    if (fs.lstatSync(this.selectedUriFolder.fsPath + this.pathSeparator + ".openapi-generator-ignore").isFile()) {
                        const response = yield vscode.window.showWarningMessage(Translator_1.default.getString("wizard.promptToOverwrite"), { modal: true }, Translator_1.default.getString("wizard.yes"));
                        if (response === Translator_1.default.getString("wizard.yes")) {
                            resolve(Constants_1.default.OPENAPI_GENERATOR_IGNORE_FILE_EXISTS); // Ok, to continue to generate
                        }
                        else {
                            reject("Overwrite files was declined"); // For log
                        }
                    }
                    else {
                        reject("Unexpected folder .openapi-generator-ignore exists."); // It's a folder.  This should not expected
                    }
                }
                catch (e) {
                    // .openapi-generator-ignore file does not exist, so should be ok to generate without overwriting files
                    resolve();
                }
            }));
        });
    }
    callGenerator() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                var docker = new dockerode();
                yield this.checkDockerImage(docker).then((result) => __awaiter(this, void 0, void 0, function* () {
                    this.imageExists = true;
                    Logger_1.default.i("The image has already been pulled. Expected true. Got: " + result);
                })).catch((result) => __awaiter(this, void 0, void 0, function* () {
                    this.imageExists = false;
                    Logger_1.default.i("The image has not been pulled. Expected false.  Got:" + result);
                }));
                if (!this.imageExists) {
                    const promptToPull = yield vscode.window.showWarningMessage(Translator_1.default.getString("wizard.promptToPullImage"), { modal: true }, Translator_1.default.getString("wizard.yes"));
                    if (promptToPull === Translator_1.default.getString("wizard.yes")) {
                        yield vscode.window.withProgress({
                            location: vscode.ProgressLocation.Notification,
                            title: Translator_1.default.getString("wizard.pullingImage"),
                            cancellable: false
                        }, (progress) => __awaiter(this, void 0, void 0, function* () {
                            return new Promise((resolve, reject) => {
                                this.pullDockerImage(docker, progress).then(() => __awaiter(this, void 0, void 0, function* () {
                                    yield this.doGeneration(progress).then(() => __awaiter(this, void 0, void 0, function* () {
                                        resolve();
                                    })).catch(() => __awaiter(this, void 0, void 0, function* () {
                                        reject();
                                    }));
                                })).catch(() => {
                                    reject();
                                });
                            });
                        }));
                    }
                    else { // If no to pull image, then return. Perhaps prompt to use Java ?
                        reject("Prompt to pull image was declined"); // For log
                        return;
                    }
                }
                else {
                    yield vscode.window.withProgress({
                        location: vscode.ProgressLocation.Notification,
                        title: Translator_1.default.getString("wizard.generatorRunning"),
                        cancellable: false
                    }, (progress) => __awaiter(this, void 0, void 0, function* () {
                        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                            yield this.doGeneration(progress).then(() => __awaiter(this, void 0, void 0, function* () {
                                resolve();
                            })).catch(() => __awaiter(this, void 0, void 0, function* () {
                                reject();
                            }));
                        }));
                    }));
                }
                resolve("");
            }));
        });
    }
    enableProgressReporter(progress) {
        return __awaiter(this, void 0, void 0, function* () {
            var outStr = new stream_1.Stream.PassThrough();
            return new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                outStr.on('data', (data) => __awaiter(this, void 0, void 0, function* () {
                    const result = data.toString();
                    Logger_1.default.i(`${result}`);
                    var index = result.indexOf('writing file'); // Do not externalize to resource file
                    if (index > 0) {
                        progress.report({ message: result.substr(index) });
                    }
                    else {
                        progress.report({ message: `${result}` });
                    }
                }));
                resolve(outStr);
            }));
        });
    }
    isChildFolder(initialPath, fqPath) {
        var index = fqPath.lastIndexOf(initialPath);
        if (process.platform.toLowerCase().startsWith("win")) {
            index = fqPath.toLowerCase().lastIndexOf(initialPath.toLowerCase());
        }
        return (index === 0);
    }
    getFilenameFromPath(initialPath, fqPath) {
        if (this.isChildFolder(initialPath, fqPath)) {
            return fqPath.substr(initialPath.length + 1); // ends with a slash
        }
        return "";
    }
    getPlatformPath(fqPath) {
        var adjustedPath = fqPath;
        if (process.platform.toLowerCase().startsWith("win")) {
            if (fqPath.charAt(2) === ':') {
                adjustedPath = fqPath.substr(0, 2) + fqPath.substr(3);
            }
        }
        return adjustedPath;
    }
    getPathSeparator() {
        if (process.platform.toLowerCase().startsWith("win")) {
            return "\\";
        }
        return "/";
    }
    // No built-in way to do content checking
    isPotentialOpenApiFile(name) {
        var adjustedName = name.toLowerCase();
        if (adjustedName.startsWith('.')
            || adjustedName.endsWith('package.json')
            || adjustedName.endsWith('package-lock.json')
            || adjustedName.endsWith('chart.yaml')
            || adjustedName.endsWith('nodemon.json')
            || adjustedName.endsWith('manifest.yml')
            || adjustedName.endsWith('tslint.json')
            || adjustedName.endsWith('devfile.yaml')) {
            return false;
        }
        return true; // Return true 'blindly' accepting that it is an OpenAPI document
    }
}
exports.default = AbstractGenerateCommand;
//# sourceMappingURL=AbstractGenerateCommand.js.map