"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
const vscode = __importStar(require("vscode"));
const Constants_1 = __importDefault(require("../constants/Constants"));
const Logger_1 = __importDefault(require("../util/Logger"));
const Utils_1 = __importDefault(require("../util/Utils"));
const AbstractGenerateCommand_1 = __importDefault(require("./AbstractGenerateCommand"));
const Translator_1 = __importDefault(require("../constants/Translator"));
const PomUtilities_1 = __importDefault(require("../util/PomUtilities"));
var dockerode = require('dockerode');
var fs = require('fs');
var reqPath = require('path');
var copydir = require('copy-dir');
class AbstractGenerateStubCommand extends AbstractGenerateCommand_1.default {
    constructor(_generatorType) {
        super(_generatorType);
        this.pomFileExists = false;
        this.selectedGeneratorType = "";
    }
    generate(selection) {
        return __awaiter(this, void 0, void 0, function* () {
            // reset values before generation
            this.projectLanguage = "";
            this.selectedGeneratorType = "";
            this.projectName = "";
            this.localPath = vscode.Uri.parse("file://");
            this.imageExists = false;
            this.selectedDefinition = "";
            this.fqPathToDefinition = "";
            this.fqPathOutputLocation = "";
            var currentGeneratorTypes = [];
            try {
                // Check if the command is launched from codewind's view and the context is a known Microclimate project
                if (selection !== undefined) {
                    yield this.analyzeSelection(selection);
                }
                // If there is no project context then prompt for target project and target language
                if (this.projectName === "") {
                    yield this.promptForProject();
                }
                // scan the project for OpenAPI definitions
                if (this.projectName !== "") {
                    yield this.promptForOpenApiDefinition();
                    if (this.projectLanguage === "" || this.projectLanguage === "unknown") { // If project language is still not determined, we must prompt the user
                        var langTypeMap = Constants_1.default.ALL_CLIENT_LANGUAGES;
                        if (this._generatorType === "server") {
                            langTypeMap = Constants_1.default.ALL_SERVER_LANGUAGES;
                        }
                        var allLangs = [];
                        var len = langTypeMap.length;
                        for (var idx = 0; idx < len; idx++) {
                            var a = langTypeMap[idx][0];
                            allLangs.push(a);
                        }
                        yield vscode.window.showQuickPick(allLangs, Constants_1.default.QUICK_PICK_OPTION_LANGUAGES).then((selectedLanguage) => __awaiter(this, void 0, void 0, function* () {
                            if (selectedLanguage === undefined) {
                                return;
                            }
                            this.projectLanguage = selectedLanguage;
                        }));
                    }
                    switch (this.projectLanguage) {
                        case "go":
                        case "Go":
                            currentGeneratorTypes = this.goTypes;
                            break;
                        case "java":
                        case "Java":
                            if ("spring" === this.projectType) {
                                currentGeneratorTypes = this.javaSpringTypes;
                            }
                            else if (this.projectType.toLowerCase().indexOf("liberty") >= 0 ||
                                this.projectType.toLowerCase().indexOf("docker") >= 0) {
                                currentGeneratorTypes = this.codewindTypes;
                            }
                            else if (this.projectType.toLowerCase().indexOf("appsodyExtension") >= 0) { // appsodyExtension
                                currentGeneratorTypes = this.codewindTypes;
                                currentGeneratorTypes = this.javaTypes.concat(this.javaSpringTypes);
                            }
                            else { // Default to any java and spring types
                                currentGeneratorTypes = this.javaTypes;
                                currentGeneratorTypes = this.javaTypes.concat(this.javaSpringTypes);
                            }
                            break;
                        case "nodejs":
                        case "Node.js":
                            currentGeneratorTypes = this.nodeTypes;
                            break;
                        case "swift":
                        case "Swift":
                            currentGeneratorTypes = this.swiftTypes;
                            this.preferredSourceLocation = "/Sources";
                            break;
                        case "python":
                        case "Python":
                            currentGeneratorTypes = this.pythonTypes;
                            break;
                        default:
                            var langs = Constants_1.default.ALL_CLIENT_LANGUAGES;
                            if (this._generatorType === "server") {
                                langs = Constants_1.default.ALL_SERVER_LANGUAGES;
                            }
                            var numOfLangs = langs.length;
                            for (var inde = 0; inde < numOfLangs; inde++) {
                                var l = langs[inde][0];
                                if (l === this.projectLanguage) {
                                    currentGeneratorTypes = langs[inde][1];
                                    break;
                                }
                            }
                    }
                    if (currentGeneratorTypes.length === 0) {
                        vscode.window.showInformationMessage(Translator_1.default.getString("codeGen.infoNoGeneratorTypes", this._generatorType, this.projectLanguage));
                        return;
                    }
                    // ******* Generator Option ***********
                    this.selectedGeneratorType = "";
                    // ************************************   
                    if (currentGeneratorTypes.length === 1) {
                        this.selectedGeneratorType = currentGeneratorTypes[0];
                    }
                    else {
                        yield vscode.window.showQuickPick(currentGeneratorTypes, this.quickPickOptionGeneratorTypes).then((tempSelectedGeneratorType) => __awaiter(this, void 0, void 0, function* () {
                            Logger_1.default.i("Generator type selected: " + tempSelectedGeneratorType);
                            if (tempSelectedGeneratorType === undefined) {
                                return;
                            }
                            this.selectedGeneratorType = tempSelectedGeneratorType;
                        }));
                        if (this.selectedGeneratorType === "") {
                            return;
                        }
                    }
                    // Target source folder may be specific to project type (eg. 'Sources' folder for Swift)                
                    var response = yield this.getOutputLocation();
                    this.pomFileExists = yield fs.existsSync(this.localPath.fsPath + "/pom.xml");
                    // If the selected output location already contains generated files, then it is not necessary
                    // to back up the pom.xml if applicable
                    var fileToBeMerged = "";
                    // Skip pre-code gen if ignore file exists already.  If code generation has already been performed in the project,
                    // then the merge should have already taken place.  There is no need to merge again.
                    // If output location is the same as the project root, then back up the original pom.xml
                    if (this.selectedUriFolder.fsPath === this.localPath.fsPath &&
                        response !== Constants_1.default.OPENAPI_GENERATOR_IGNORE_FILE_EXISTS) {
                        // Knowing the 'Codewind' project type ONLY is not sufficient.  We must check whether a specific
                        // file exists.
                        // Check for an existing pom.xml and then back it up
                        var msgToShow = "wizard.existingFilesWarning";
                        if (this.projectType === "spring") {
                            msgToShow = "wizard.existingFilesWarningForSpring";
                        }
                        fileToBeMerged = yield Utils_1.default.backupFileIfExists(this.localPath.fsPath, "pom", ".xml", msgToShow);
                    }
                    //////////////////////////////////////////////////////////
                    // Workaround for Java spring server generator only
                    //////////////////////////////////////////////////////////
                    var templatePath = reqPath.join(__dirname, '/../templates/JavaSpring401');
                    if (this.selectedGeneratorType === "spring") {
                        fs.mkdirSync(this.selectedUriFolder.fsPath + "/.cwopenapitemplates");
                        yield copydir.sync(templatePath, this.selectedUriFolder.fsPath + "/.cwopenapitemplates");
                    }
                    // End of workaround /////////////////////////////////////
                    Logger_1.default.i("****** About to call generator");
                    yield this.callGenerator();
                    // Do post code gen config if chosen output location is at the project's root.
                    // Skip post-code gen if ignore file exists already.  If code generation has already been performed in the project,
                    // then the merge should have already taken place.  There is no need to merge again.
                    if (this.selectedUriFolder.fsPath === this.localPath.fsPath &&
                        response !== Constants_1.default.OPENAPI_GENERATOR_IGNORE_FILE_EXISTS &&
                        this.pomFileExists &&
                        fileToBeMerged.length > 0) {
                        yield PomUtilities_1.default.postCodeGenPomConfiguration(this.localPath.fsPath, fileToBeMerged, true);
                    }
                }
            }
            catch (error) { // Catch for all expected rejected promises or cancelled quick picks or inputs
                Logger_1.default.i("Code generation stopped or failed. " + error);
            }
        });
    }
    analyzeSelection(selection) {
        const _super = Object.create(null, {
            analyzeSelection: { get: () => super.analyzeSelection }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.analyzeSelection.call(this, selection);
            if (Reflect.has(selection, "type")) {
                this.projectLanguage = Reflect.get(selection, "type").language;
            }
            Logger_1.default.i("Selected project language is: " + this.projectLanguage);
            if (this.projectLanguage === "swift") {
                this.preferredSourceLocation = "/Sources";
            }
            if (this.projectLanguage === 'unknown') {
                this.projectLanguage = "";
                yield this.determineLanguage();
            }
        });
    }
    doGeneration(progress) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                var docker = new dockerode();
                var outStr = yield this.enableProgressReporter(progress);
                var cmdLineArgs;
                // Set up the command line args. See if-else clauses below
                // The following vars are for jaxrs-spec server generation only.
                // Codewind APIs do not give us server-specific details. It currently gives us docker or appsodyExtension project types.
                // Also, this needs to work when the commands are launched from the command palette where we don't have access to the project type information.
                // For now, simply check for the presence of the server.xml in the liberty folder in the project
                var isLiberty = yield fs.existsSync(this.localPath.fsPath + "/src/main/liberty/config/server.xml");
                var isAppsodyLiberty = isLiberty && (yield fs.existsSync(this.localPath.fsPath + "/.appsody-config.yaml"));
                //////////////////////////////////////////////////////////
                // Workaround for Java spring server generator only
                //////////////////////////////////////////////////////////
                if (this.selectedGeneratorType === "spring") {
                    var filePath = this.localPath.fsPath;
                    cmdLineArgs = ['generate', '-i', '/gen/' + this.selectedDefinition, '-g', this.selectedGeneratorType, '--enable-post-process-file', '-t', "/out/.cwopenapitemplates/", '-o', '/out'];
                    if (this.pomFileExists) {
                        cmdLineArgs.push('--additional-properties');
                        cmdLineArgs.push('mavenPomExists=true');
                        Logger_1.default.i("CmdLineArgs is " + cmdLineArgs.toString());
                        // cmdLineArgs = ['generate', '-i', '/gen/' + this.selectedDefinition, '-g', this.selectedGeneratorType, '--enable-post-process-file', '-java8', '-t', "/out/.cwopenapitemplates/", '-o', '/out', '--additional-properties', 'mavenPomExists=true'];
                        Logger_1.default.i("Mapped gen command is " + 'generate  -i /gen/' + this.selectedDefinition + ' -g ' + this.selectedGeneratorType + ' -t ' + "/out/.cwopenapitemplates/" + ' -o /out -v ' + this.fqPathToDefinition + ':/gen' + " -v " + this.fqPathOutputLocation + ':/out --additional-properties mavenPomExists=true');
                    }
                    else {
                        // cmdLineArgs = ['generate', '-i', '/gen/' + this.selectedDefinition, '-g', this.selectedGeneratorType, '--enable-post-process-file', '-java8', '-t', "/out/.cwopenapitemplates/", '-o', '/out'];
                        Logger_1.default.i("Mapped gen command is " + 'generate  -i /gen/' + this.selectedDefinition + ' -g ' + this.selectedGeneratorType + ' -t ' + "/out/.cwopenapitemplates/" + ' -o /out -v ' + this.fqPathToDefinition + ':/gen' + " -v " + this.fqPathOutputLocation + ':/out');
                    }
                }
                else if (this.selectedGeneratorType === "jaxrs-spec" && (this.projectType.toLowerCase().indexOf("liberty") >= 0 ||
                    (this.projectType.toLowerCase().indexOf("docker") >= 0 && isLiberty) ||
                    (this.projectType.toLowerCase().indexOf("appsodyextension") >= 0 && isAppsodyLiberty))) {
                    cmdLineArgs = ['generate', '-i', '/gen/' + this.selectedDefinition, '-g', this.selectedGeneratorType, '--library', 'openliberty', '--enable-post-process-file', '-o', '/out'];
                    Logger_1.default.i("Mapped jax-rs spec gen command is " + 'generate -i /gen/' + this.selectedDefinition + ' -g ' + this.selectedGeneratorType + '--library openliberty' + ' -o /out -v ' + this.fqPathToDefinition + ':/gen' + " -v " + this.fqPathOutputLocation + ':/out');
                }
                else {
                    cmdLineArgs = ['generate', '-i', '/gen/' + this.selectedDefinition, '-g', this.selectedGeneratorType, '--enable-post-process-file', '-o', '/out'];
                    Logger_1.default.i("Mapped gen command is " + 'generate -i /gen/' + this.selectedDefinition + ' -g ' + this.selectedGeneratorType + ' -o /out -v ' + this.fqPathToDefinition + ':/gen' + " -v " + this.fqPathOutputLocation + ':/out');
                }
                // End of workaround /////////////////////////////////////
                yield docker.run(this.openapiGeneratorImage, cmdLineArgs, outStr, { HostConfig: { "Binds": [`${this.fqPathToDefinition}:/gen`, `${this.fqPathOutputLocation}:/out`] } }, {}, (err, data, ctnr) => __awaiter(this, void 0, void 0, function* () {
                    try {
                        ctnr.remove();
                        //////////////////////////////////////////////////////////
                        // Workaround for Java spring server generator only
                        // Remove when template code is fixed in openapi-generator                        
                        if (this.selectedGeneratorType === "spring") {
                            Utils_1.default.removeDir(this.selectedUriFolder.fsPath + "/.cwopenapitemplates");
                        }
                        // End of workaround /////////////////////////////////////
                        if (err) {
                            Logger_1.default.e("docker.run error is: " + err);
                            vscode.window.showErrorMessage(Translator_1.default.getString("codeGen.failedToGenerateStub", this.selectedGeneratorType, err));
                            reject();
                        }
                        else {
                            if (data !== null) {
                                Logger_1.default.i("data.StatusCode = " + data.StatusCode);
                                Logger_1.default.i("data.Error = " + data.Error);
                                if (data.StatusCode === 0) {
                                    vscode.window.showInformationMessage(Translator_1.default.getString("codeGen.success", this.selectedGeneratorType));
                                    resolve();
                                }
                                else { // Other issues?
                                    vscode.window.showInformationMessage(Translator_1.default.getString("codeGen.failedNonZeroStatus", this.selectedGeneratorType, data.StatusCode, data.Error));
                                    reject();
                                }
                            }
                            else {
                                reject();
                            }
                        }
                    }
                    catch (e) {
                        reject();
                    }
                }));
            }));
        });
    }
    determineLanguage() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.projectLanguage === "") { // If project language is still not determined, we must prompt the user
                var langTypeMap = Constants_1.default.ALL_CLIENT_LANGUAGES;
                if (this._generatorType === "server") {
                    langTypeMap = Constants_1.default.ALL_SERVER_LANGUAGES;
                }
                var allLangs = [];
                var len = langTypeMap.length;
                for (var idx = 0; idx < len; idx++) {
                    var a = langTypeMap[idx][0];
                    allLangs.push(a);
                }
                yield vscode.window.showQuickPick(allLangs, Constants_1.default.QUICK_PICK_OPTION_LANGUAGES).then((selectedLanguage) => {
                    if (selectedLanguage === undefined) {
                        return;
                    }
                    this.projectLanguage = selectedLanguage;
                });
            }
        });
    }
}
exports.default = AbstractGenerateStubCommand;
//# sourceMappingURL=AbstractGenerateStubCommand.js.map