"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
const vscode = __importStar(require("vscode"));
const Logger_1 = __importDefault(require("../util/Logger"));
const AbstractGenerateCommand_1 = __importDefault(require("./AbstractGenerateCommand"));
const Translator_1 = __importDefault(require("../constants/Translator"));
var dockerode = require('dockerode');
class HtmlDocsCommand extends AbstractGenerateCommand_1.default {
    constructor(_generatorType) {
        super(_generatorType);
    }
    generate(selection) {
        return __awaiter(this, void 0, void 0, function* () {
            this.projectName = "";
            this.localPath = vscode.Uri.parse("file://");
            this.imageExists = false;
            this.selectedDefinition = "";
            this.fqPathToDefinition = "";
            this.fqPathOutputLocation = "";
            try {
                // Check if the command is launched from codewind's view and the context is a known Microclimate project
                if (selection !== undefined) {
                    yield this.analyzeSelection(selection);
                }
                // If there is no project context then prompt for target project
                if (this.projectName === "") {
                    yield this.promptForProject();
                }
                // scan the project for OpenAPI definitions
                if (this.projectName !== "") {
                    yield this.promptForOpenApiDefinition();
                    yield this.getOutputLocation();
                    yield this.callGenerator();
                }
            }
            catch (errorOrMessage) {
                Logger_1.default.i("HTML generation stopped or failed. " + errorOrMessage);
            }
        });
    }
    doGeneration(progress) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                var docker = new dockerode();
                var outStr = yield this.enableProgressReporter(progress);
                Logger_1.default.i("Mapped gen comamnd is " + 'generate -i /gen/' + this.selectedDefinition + ' -g ' + 'html2' + ' -o /out -v ' + this.fqPathToDefinition + ':/gen' + " -v " + this.fqPathOutputLocation + ':/out');
                yield docker.run(this.openapiGeneratorImage, ['generate', '-i', '/gen/' + this.selectedDefinition, '-g', 'html2', '-o', '/out'], outStr, { HostConfig: { "Binds": [`${this.fqPathToDefinition}:/gen`, `${this.fqPathOutputLocation}:/out`] } }, {}, (err, data, ctnr) => __awaiter(this, void 0, void 0, function* () {
                    ctnr.remove();
                    if (err) {
                        Logger_1.default.e("docker.run error is: " + err);
                        vscode.window.showErrorMessage(Translator_1.default.getString("docGen.failedToGenerateHtml", err));
                        reject();
                    }
                    else {
                        if (data !== null) {
                            Logger_1.default.i("data.StatusCode = " + data.StatusCode);
                            Logger_1.default.i("data.Error = " + data.Error);
                            if (data.StatusCode === 0) {
                                vscode.window.showInformationMessage(Translator_1.default.getString("docGen.success"));
                                resolve();
                            }
                            else { // For other issues?
                                vscode.window.showInformationMessage(Translator_1.default.getString("docGen.failedNonZeroStatus", data.StatusCode, data.Error));
                                reject();
                            }
                        }
                        else {
                            reject();
                        }
                    }
                }));
            }));
        });
    }
}
exports.default = HtmlDocsCommand;
//# sourceMappingURL=HtmlDocsCommand.js.map