"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = __importDefault(require("../util/Logger"));
const Translator_1 = __importDefault(require("./Translator"));
/*
CLIENT generators:
    - ada
    - android
    - apex
    - bash
    - c
    - clojure
    - cpp-qt5-client
    - cpp-restsdk
    - cpp-tizen
    - csharp
    - csharp-dotnet2
    - csharp-netcore
    - dart
    - dart-jaguar
    - eiffel
    - elixir
    - elm
    - erlang-client
    - erlang-proper
    - flash
    - go
    - groovy
    - haskell-http-client
    - java
    - javascript
    - javascript-closure-angular
    - javascript-flowtyped
    - jaxrs-cxf-client
    - jmeter
    - kotlin
    - lua
    - objc
    - perl
    - php
    - powershell
    - python
    - r
    - ruby
    - rust
    - scala-akka
    - scala-gatling
    - scala-httpclient-deprecated
    - scalaz
    - swift2-deprecated
    - swift3-deprecated
    - swift4
    - typescript-angular
    - typescript-angularjs
    - typescript-aurelia
    - typescript-axios
    - typescript-fetch
    - typescript-inversify
    - typescript-jquery
    - typescript-node
    - typescript-rxjs


SERVER generators:
    - ada-server
    - aspnetcore
    - cpp-pistache-server
    - cpp-qt5-qhttpengine-server
    - cpp-restbed-server
    - csharp-nancyfx
    - erlang-server
    - go-gin-server
    - go-server
    - graphql-nodejs-express-server
    - haskell
    - java-inflector
    - java-msf4j
    - java-pkmst
    - java-play-framework
    - java-undertow-server
    - java-vertx
    - jaxrs-cxf
    - jaxrs-cxf-cdi
    - jaxrs-cxf-extended
    - jaxrs-jersey
    - jaxrs-resteasy
    - jaxrs-resteasy-eap
    - jaxrs-spec
    - kotlin-server
    - kotlin-spring
    - nodejs-express-server, nodejs-server-deprecated
    - php-laravel
    - php-lumen
    - php-silex
    - php-slim
    - php-symfony
    - php-ze-ph
    - python-aiohttp
    - python-blueplanet
    - python-flask
    - ruby-on-rails
    - ruby-sinatra
    - rust-server
    - scala-finch
    - scala-lagom-server
    - scala-play-server
    - scalatra
    - spring


DOCUMENTATION generators:
    - cwiki
    - dynamic-html
    - html
    - html2
    - openapi
    - openapi-yaml
*/
class Constants {
}
exports.default = Constants;
// Based on package.json
Constants.EXTENSION_ID = "IBM.codewind-openapi-tools";
Constants.PRIMARY_LANGUAGES = ['Go', 'Java', 'Node.js', 'Python', 'Swift'];
Constants.ALL_CLIENT_LANGUAGES = [
    ['Go', ['go']],
    ['Java', ['java', 'jaxrs-cxf-client']],
    ['Node.js', ['javascript',
            'javascript-closure-angular',
            'javascript-flowtyped',
            'typescript-angular',
            'typescript-angularjs',
            'typescript-aurelia',
            'typescript-axios',
            'typescript-fetch',
            'typescript-inversify',
            'typescript-jquery',
            'typescript-node']],
    ['Python', ['python']],
    ['Swift', ['swift3', 'swift4', 'swift2-deprecated']],
    ['Ada', ['ada']],
    ['Apex', ['apex']],
    ['Bash', ['bash']],
    ['C', ['c']],
    ['C#', ['csharp', 'csharp-dotnet2', 'csharp-refactor', 'csharp-netcore']],
    ['C++', ['cpp-qt5', 'cpp-restsdk', 'cpp-tizen']],
    ['Dart', ['dart', 'dart-jaguar']],
    ['Eiffel', ['eiffel']],
    ['Elixir', ['elixir']],
    ['Elm', ['elm']],
    ['Erlang', ['erlang-client', 'erlang-proper']],
    ['Haskell', ['haskell-http-client']],
    ['Kotlin', ['kotlin']],
    ['Lua', ['lua']],
    ['Objective-C', ['objc']],
    ['Perl', ['perl']],
    ['PHP', ['php']],
    ['PowerShell', ['powershell']],
    ['R', ['r']],
    ['Ruby', ['ruby']],
    ['Rust', ['rust']],
    ['Scala', ['scala-akka', 'scala-gatling', 'scala-httpclient', 'scalaz']],
    ['TypeScript', ['typescript-angular',
            'typescript-angularjs',
            'typescript-aurelia',
            'typescript-axios',
            'typescript-fetch',
            'typescript-inversify',
            'typescript-jquery',
            'typescript-node']]
];
Constants.ALL_SERVER_LANGUAGES = [
    ['Go', ['go-gin-server', 'go-server']],
    ['Java', ['jaxrs-spec',
            'java-inflector',
            'java-msf4j',
            'java-pkmst',
            'java-play-framework',
            'java-undertow-server',
            'java-vertx',
            'jaxrs-cxf',
            'jaxrs-cxf-cdi',
            'jaxrs-jersey',
            'jaxrs-resteasy',
            'jaxrs-resteasy-eap',
            'spring']],
    ['Node.js', ['nodejs-express-server', 'nodejs-server-deprecated']],
    ['Python', ['python-flask']],
    ['Swift', []],
    ['Ada', ['ada-server']],
    ['C#', ['csharp-nancyfx']],
    ['C++', ['cpp-pistache-server', 'cpp-qt5-qhttpengine-server', 'cpp-restbed-server']],
    ['Erlang', ['erlang-server']],
    ['Haskell', ['haskell']],
    ['Kotlin', ['kotlin-server', 'kotlin-spring']],
    ['PHP', ['php-laravel', 'php-lumen', 'php-silex', 'php-slim', 'php-symfony', 'php-ze-ph']],
    ['Python', ['python-flask']],
    ['Ruby', ['ruby-on-rails', 'ruby-sinatra']],
    ['Rust', ['rust-server']],
    ['Scala', ['scala-finch', 'scala-lagom-server', 'scalatra']],
];
Constants.vvv = "Project";
Constants.QUICK_PICK_OPTION_PROJECT_FOLDERS = {
    matchOnDescription: true,
    matchOnDetail: true,
    canPickMany: false,
    placeHolder: Translator_1.default.getString("commands.selectProject"),
    onDidSelectItem: (item) => Logger_1.default.i(`Item ${item} is selected`)
};
Constants.QUICK_PICK_OPTION_OPENAPI_DEFINITIONS = {
    matchOnDescription: true,
    matchOnDetail: true,
    canPickMany: false,
    placeHolder: Translator_1.default.getString("commands.selectOpenApiDefinition"),
    onDidSelectItem: (item) => Logger_1.default.i(`Item ${item} is selected`)
};
Constants.QUICK_PICK_OPTION_LANGUAGES = {
    matchOnDescription: true,
    matchOnDetail: true,
    canPickMany: false,
    placeHolder: Translator_1.default.getString("commands.selectPrimaryLanguage"),
    onDidSelectItem: (item) => Logger_1.default.i(`Item ${item} is selected`)
};
Constants.QUICK_PICK_OPTION_CLIENT_TYPES = {
    matchOnDescription: true,
    matchOnDetail: true,
    canPickMany: false,
    placeHolder: Translator_1.default.getString("commands.selectClientGenType"),
    onDidSelectItem: (item) => Logger_1.default.i(`Item ${item} is selected`)
};
Constants.QUICK_PICK_OPTION_SERVER_TYPES = {
    matchOnDescription: true,
    matchOnDetail: true,
    canPickMany: false,
    placeHolder: Translator_1.default.getString("commands.selectServerGenType"),
    onDidSelectItem: (item) => Logger_1.default.i(`Item ${item} is selected`)
};
Constants.OPENAPI_GENERATOR_IGNORE_FILE_EXISTS = "OPENAPI_GENERATOR_IGNORE_FILE_EXISTS";
//# sourceMappingURL=Constants.js.map