"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
const fs = __importStar(require("fs"));
const os = __importStar(require("os"));
const path = __importStar(require("path"));
const util = __importStar(require("util"));
const vscode_1 = require("vscode");
const Stacktrace = __importStar(require("stack-trace"));
const CircularJson = __importStar(require("circular-json"));
// non-nls-file
// tslint:disable no-console
class Log {
    static get getLogFilePath() {
        return this.logFilePath;
    }
    static setLogFilePath(context) {
        // Directory provided by extension context may not exist
        const logDir = context.logPath;
        const mode = 0o744;
        try {
            fs.accessSync(logDir, mode);
        }
        catch (err) {
            // logDir doesn't exist, we must create it
            try {
                fs.mkdirSync(logDir, mode);
                console.log("Codewind OpenAPI Tools created logs dir", logDir);
            }
            catch (err) {
                // This shouldn't happen, but fall back to console.log if it does.
                console.error("Error creating log file!", err);
                this.logInner = util.promisify(console.log);
            }
        }
        const fullPath = path.join(context.logPath, this.LOG_NAME);
        this.logFilePath = fullPath;
        console.log("Codewind OpenAPI Tools log file is at " + this.logFilePath);
        this.i("Codewind OpenAPI Tools Logger initialized at " + this.logFilePath);
    }
    static silenceLevels(level, ...levels) {
        levels = levels.concat(level);
        Log.i("Disabling log levels:", levels);
        this.disabledLevels = levels;
    }
    static d(s, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logInner(Log.Levels.DEBUG, s, args);
        });
    }
    static i(s, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logInner(Log.Levels.INFO, s, args);
        });
    }
    static w(s, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logInner(Log.Levels.WARNING, s, args);
        });
    }
    static e(s, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logInner(Log.Levels.ERROR, s, args);
        });
    }
    static t(s, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logInner(Log.Levels.TEST, s, args);
        });
    }
    static logInner(level, s, args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.logFilePath === null) {
                console.error("Logger.log error - No log file path set!");
                console.log(s, args);
                return;
            }
            else if (this.disabledLevels.includes(level)) {
                return;
            }
            const argsStr = args.reduce((result, arg) => {
                if (arg instanceof Object) {
                    try {
                        // Can fail eg on objects with circular references
                        // arg = JSON.stringify(arg, undefined, 2);
                        arg = CircularJson.stringify(arg, replacer, 2);
                    }
                    catch (err) {
                        if (err.message && err.message.includes("circular")) {
                            arg = "*** Couldn't stringify circular object";
                        }
                        else {
                            arg = `*** Failed to log object`;
                        }
                    }
                }
                result = result.concat(os.EOL, arg);
                return result;
            }, s);
            let caller = "";
            try {
                caller = " " + this.getCaller();
            }
            catch (err) {
                console.error(err);
            }
            const label = `[${level}: ${getDateTime()}${caller}]:`;
            const msg = `${label} ${argsStr}${os.EOL}`;
            return new Promise((resolve) => {
                // Send the message to both the 'console' and the logfile.
                const consoleFn = level === this.Levels.ERROR ? console.error : console.log;
                if (args.length > 0) {
                    consoleFn(label, s, ...args);
                }
                else {
                    consoleFn(label, s);
                }
                fs.appendFile(this.logFilePath, msg, (err) => {
                    if (err != null) {
                        console.error("FS error when logging:", err);
                    }
                    return resolve();
                });
            });
        });
    }
    static getCaller() {
        // get the stack trace above logInner. casting is necessary because the stack trace package only accepts () => void functions.
        const stack = Stacktrace.get(Log.logInner);
        // 6 frames is the magic number to get around __awaiters, past the Log.x function, and up to the frame we care about.
        const frame = stack[6];
        if (frame === null) {
            return "N/A";
        }
        let methodName = frame.getMethodName() || frame.getFunctionName();
        if (methodName !== null) {
            // If it's a callback, there will be extra stuff we aren't interested in separated by dots
            // eg "Project.__dirname.constructor.connection.update"
            // strip out everything up to the last dot, if there is one
            const splitByPeriod = methodName.split(".");
            if (splitByPeriod.length > 1) {
                methodName = splitByPeriod[splitByPeriod.length - 1];
            }
            methodName = `.${methodName}()`;
        }
        else {
            methodName = "";
        }
        const fileName = path.basename(frame.getFileName());
        return `${fileName}${methodName}:${frame.getLineNumber()}`;
    }
}
exports.Log = Log;
Log.LOG_NAME = "codewind-openapi-tools.log";
Log.disabledLevels = [];
function replacer(val) {
    if (val instanceof vscode_1.Uri) {
        return val.toString();
    }
    return val;
}
function getDateTime() {
    const now = new Date();
    // formats to eg. 22/10/2018 9:40:15.832
    // Note add 1 to month because months are 0-indexed
    // return `${leftPad(now.getDate())}/${leftPad(now.getMonth() + 1)}/${now.getFullYear()} ` +
    return `${leftPad(now.getHours(), 2)}:${leftPad(now.getMinutes(), 2)}:${leftPad(now.getSeconds(), 2)}.${leftPad(now.getMilliseconds(), 3)}`;
}
/**
 * Convert the given number to a string of at least the given length.
 * Eg:
 * leftPad(3, 2) -> "03"
 * leftPad(20, 2) -> "20"
 * leftpad(400, 2) -> "400"     (just converts to string)
 */
function leftPad(n, desiredLen) {
    const nStr = n.toString();
    const diff = desiredLen - nStr.length;
    if (diff <= 0) {
        return nStr;
    }
    return "0".repeat(diff) + nStr;
}
(function (Log) {
    let Levels;
    (function (Levels) {
        Levels["DEBUG"] = "DBUG";
        Levels["INFO"] = "INFO";
        Levels["WARNING"] = "WARN";
        Levels["ERROR"] = "ERRO";
        Levels["TEST"] = "TEST";
    })(Levels = Log.Levels || (Log.Levels = {}));
})(Log = exports.Log || (exports.Log = {}));
exports.default = Log;
//# sourceMappingURL=Logger.js.map