"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/*******************************************************************************
 * Copyright (c) 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
const vscode = __importStar(require("vscode"));
const Logger_1 = __importDefault(require("../util/Logger"));
const Translator_1 = __importDefault(require("../constants/Translator"));
var fs = require('fs');
var reqPath = require('path');
class Utils {
    static getUniqueFileName(filePath, proposedBackupName, fileExtension) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    var i = 0;
                    var tempName = proposedBackupName + fileExtension;
                    while (fs.existsSync(filePath + "/" + tempName)) {
                        i++;
                        tempName = proposedBackupName + "-" + i + fileExtension;
                    }
                    resolve(tempName);
                }
                catch (e) {
                    reject("");
                }
            }));
        });
    }
    static removeDir(filePath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (fs.existsSync(filePath)) {
                fs.readdirSync(filePath).forEach((item) => __awaiter(this, void 0, void 0, function* () {
                    const aPath = reqPath.join(filePath, item);
                    if (fs.lstatSync(aPath).isDirectory()) {
                        yield this.removeDir(aPath);
                    }
                    else {
                        fs.unlinkSync(aPath);
                    }
                }));
                fs.rmdirSync(filePath);
            }
        });
    }
    // General method to backup a file.  eg. pom.xml, package.json, or even source code if need be.
    // filePath: vsCode.Uri.fsPath to the resource
    // fileToBackup: name part of the file to backup excluding the extension
    // fileExtension: extension of the file, including the period
    // msgToShow: the Translator.getString key message to inform the user. This should contain only one substitution for the file name
    static backupFileIfExists(filePath, fileToBackup, fileExtension, msgToShow) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                if (yield fs.existsSync(filePath + "/" + fileToBackup + fileExtension)) {
                    yield Logger_1.default.i("Backing up original file " + fileToBackup + fileExtension);
                    if (msgToShow.length > 0) {
                        const response = yield vscode.window.showWarningMessage(Translator_1.default.getString(msgToShow, fileToBackup + fileExtension), { modal: true }, Translator_1.default.getString("wizard.yes"));
                        if (response !== Translator_1.default.getString("wizard.yes")) {
                            reject("Declined to proceed with code generation when output folder already contains the file " + fileToBackup + fileExtension);
                            return;
                        }
                    }
                    var tempName = yield Utils.getUniqueFileName(filePath, fileToBackup + "-backup", fileExtension);
                    yield fs.rename(filePath + "/" + fileToBackup + fileExtension, filePath + "/" + tempName, (e) => __awaiter(this, void 0, void 0, function* () {
                        if (e && msgToShow.length > 0) {
                            vscode.window.showInformationMessage(Translator_1.default.getString("wizard.failedToBackupFile", fileToBackup + fileExtension, tempName));
                            reject("Failed to backup file:" + fileToBackup + fileExtension);
                        }
                    }));
                    resolve(tempName);
                }
                resolve("");
            }));
        });
    }
}
exports.default = Utils;
//# sourceMappingURL=Utils.js.map